
#ifndef __CWX_LIB__
#define __CWX_LIB__




/////////////////////////////////////////////////////////////////////////////
// CXHyperLink window

class CXHyperLink : public CStatic
{
// Construction/destruction
public:
    CXHyperLink();
    virtual ~CXHyperLink();

public:
    enum UnderLineOptions { ulHover = -1, ulNone = 0, ulAlways = 1};

// Attributes
public:
    void SetURL(CString strURL);
	CString GetURL() const
	{ 
		return m_strURL; 
	}

	void EnableURL(BOOL bFlag) { m_bIsURLEnabled = bFlag; }
	BOOL IsURLEnabled() const { return m_bIsURLEnabled; }

    void SetColours(COLORREF crLinkColour, 
					COLORREF crVisitedColour, 
                    COLORREF crHoverColour = -1);

	COLORREF GetLinkColour() const
	{ 
		return m_crLinkColour; 
	}

	COLORREF GetVisitedColour() const
	{
		return m_crVisitedColour; 
	}

	COLORREF GetHoverColour() const
	{
		return m_crHoverColour;
	}

	void SetBackgroundColour(COLORREF crBackground);
	COLORREF GetBackgroundColour() const
	{
		return m_crBackground;
	}

    void SetVisited(BOOL bVisited = TRUE);
	BOOL GetVisited() const
	{ 
		return m_bVisited; 
	}

    void SetLinkCursor(HCURSOR hCursor);
	HCURSOR CXHyperLink::GetLinkCursor() const
	{
		return m_hLinkCursor;
	}

    void SetUnderline(int nUnderline = ulHover);
	int GetUnderline() const
	{ 
		return m_nUnderline; 
	}

    void SetAutoSize(BOOL bAutoSize = TRUE);
	BOOL GetAutoSize() const
	{ 
		return m_bAdjustToFit; 
	}

	void SetNotifyParent(BOOL bFlag) { m_bNotifyParent = bFlag; }
	BOOL GetNotifyParent() const { return m_bNotifyParent; }

	void EnableTooltip(BOOL bFlag) 
	{ 
		m_bToolTip = bFlag; 
		m_ToolTip.Activate(m_bToolTip);
	}
	BOOL IsTooltipEmabled() const 
	{ 
		return m_bToolTip; 
	}

	void SetAlwaysOpenNew(BOOL bFlag) 
	{ 
		m_bAlwaysOpenNew = bFlag; 
	}
	BOOL GetAlwaysOpenNew() const 
	{ 
		return m_bAlwaysOpenNew; 
	}

	void SetWindowText(LPCTSTR lpszString);


// Operations
public:
	static HINSTANCE GotoURL(LPCTSTR url, int showcmd,  BOOL bAlwaysOpenNew = FALSE);


// Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CXHyperLink)
public:
    virtual BOOL PreTranslateMessage(MSG* pMsg);
	virtual BOOL DestroyWindow();
protected:
    virtual void PreSubclassWindow();
	//}}AFX_VIRTUAL

// Implementation
protected:
	static LONG GetRegKey(HKEY key, LPCTSTR subkey, LPTSTR retdata);
	void NotifyParent();
	void PositionWindow();
	void ReportError(int nError);
	void SetDefaultCursor();

// Protected attributes
protected:
    COLORREF		m_crLinkColour;		// Hyperlink colours
	COLORREF		m_crVisitedColour;
    COLORREF		m_crHoverColour;	// Hover colour
	COLORREF		m_crBackground;		// background color
	CBrush			m_Brush;			// background brush
    BOOL			m_bOverControl;		// cursor over control?
    BOOL			m_bVisited;			// Has it been visited?
    int				m_nUnderline;		// underline hyperlink?
    BOOL			m_bAdjustToFit;		// Adjust window size to fit text?
    CString			m_strURL;			// hyperlink URL
    CFont			m_UnderlineFont;	// Font for underline display
    CFont			m_StdFont;			// Standard font
    HCURSOR			m_hLinkCursor;		// Cursor for hyperlink
    CToolTipCtrl	m_ToolTip;			// The tooltip
    UINT			m_nTimerID;
	BOOL			m_bIsURLEnabled;	// TRUE = navigate to url
	BOOL			m_bNotifyParent;	// TRUE = notify parent
	BOOL			m_bToolTip;			// TRUE = display tooltip
	BOOL			m_bAlwaysOpenNew;	// TRUE = always open new browser window

    // Generated message map functions
protected:
    //{{AFX_MSG(CXHyperLink)
    afx_msg HBRUSH CtlColor(CDC* pDC, UINT nCtlColor);
    afx_msg BOOL OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);
    afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	//}}AFX_MSG
	afx_msg void OnTimer(UINT nIDEvent);
    afx_msg void OnClicked();
    DECLARE_MESSAGE_MAP()
};


#define WM_COMM_RXCHAR    WM_USER+7 // A character was received and placed in the input buffer. 
class CSerialPort
{               
public:
 int m_nWriteSize;
 void ClosePort();
 // contruction and destruction
 CSerialPort();
 virtual  ~CSerialPort();
 // port initialisation           
 BOOL  InitPort(CWnd* pPortOwner, UINT portnr = 1, UINT baud = 19200, char parity = 'N', UINT databits = 8, UINT stopbits = 1, DWORD dwCommEvents = EV_RXCHAR, UINT writebuffersize = 1024);
 HANDLE    m_hComm;
 // start/stop comm watching
 BOOL  StartMonitoring();
 BOOL  RestartMonitoring();
 BOOL  StopMonitoring();
 DWORD  GetWriteBufferSize();
 DWORD  GetCommEvents();
 DCB   GetDCB();
 void  WriteToPort(char* string);
 void  WriteToPort(char* string,int n);
 void  WriteToPort(LPCTSTR string);
 void  WriteToPort(LPCTSTR string,int n);
 //
 void  WriteMByte(char* string,int n);
 BOOL  ReceiveChar(char* ch);
 
 
 int String2Hex(CString str, char* pdata);
 int Hex2String(CString & str, char* senddata,int len);
protected:
	// protected memberfunctions
	void Check(void);
	char ConvertHexChar(char ch);
	void  ProcessErrorMessage(char* ErrorText);
	static UINT CommThread(LPVOID pParam);
	static void ReceiveChar(CSerialPort* port, COMSTAT comstat);
	static void WriteChar(CSerialPort* port);
	// thread
	CWinThread*   m_Thread;
	// synchronisation objects
	CRITICAL_SECTION m_csCommunicationSync;
	BOOL    m_bThreadAlive;
	// handles
	HANDLE    m_hWriteEvent;
	HANDLE    m_hShutdownEvent;
	// Event array. 
	// One element is used for each event. There are two event handles for each port.
	// A Write event and a receive character event which is located in the overlapped structure (m_ov.hEvent).
	// There is a general shutdown when the port is closed. 
	HANDLE    m_hEventArray[3];
	// structures
	OVERLAPPED   m_ov;
	COMMTIMEOUTS  m_CommTimeouts;
	DCB     m_dcb;
	// owner window
	CWnd*    m_pOwner;
	// misc
	UINT    m_nPortNr;
	char*    m_szWriteBuffer;
	DWORD    m_dwCommEvents;
	DWORD    m_nWriteBufferSize;
};

#endif //__CWX_LIB__
